/**********************************************************
	M40xDef.h header file for both DLL & Application
	Lawson Labs, Inc.
	3217 Phoenixville Pike
	Malvern, PA  19355
	800 321-5355 or 610 725-8800
	http://www.lawsonlabs.com
***********************************************************/ 
#ifndef _M40XDEFH_
#define _M40XDEFH_

#include <afxsock.h>

//each data block max. data points
#define		MAX_DATA_POINTS		255	
//for standard derviation calculation
#define		MAX_SDGROUP			255		
#define		MAX_CHANNELS		8

//DO NOT MODIFY THE ABOVE CONSTANTS

#define		CvntChanID(id)(id > 1 ? id + 4 : id)

typedef		void (CALLBACK* LPCALLBACKFUNC)(void);
/*******************************************************************************
 * Error code definition                                                       *
   RETURN_CODE is defined as enum, if application needs to show 
   RETURN_CODE as a error string, you can copy this block to your
   application as a string array, to ensure the correct error messge
   the order of the string array MUST BE match fellowing enum definition
*******************************************************************************/
typedef enum{
	COMMAND_OK	= 0,
	CALIBRATION_ERROR,
	ECHO_ERROR,
	INIT_FAILURE,
	INVALID_ARGUMENT,
	RESET_FAILURE,
	TIME_OUT,
	CHKSUM_ERROR,
	SET_RATE_FAILURE,
	CHECK_POWER,
	INVALID_COMMAND,
	NO_ECHO_COMMAND,
	SOCKET_SEND_ERROR,
	PENDING,
	OUT_OF_SYNC,
	FLUSH_TIME_OUT,
	FEWER_BYTES_RECVD,
	SOCKET_EVENT_FAIL,	
	INVALID_STATE,
	RESERVED1,
	RESERVED2,
	RESERVED3,
	RESERVED4,
	NOT_CONNECTED	
}RETURN_CODE;

/*******************************************************************************
 * structure definition                                                       *
*******************************************************************************/
struct SSCAN_VOLTS_BUFF{
	double	fVolts[MAX_CHANNELS][MAX_DATA_POINTS];	//volts in double format
	double	fVoltsMax[MAX_CHANNELS];				//max volts	
	double	fVoltsMin[MAX_CHANNELS];				//min. volts
	double	fVoltsAvgTmp[MAX_CHANNELS][255];		//average tmp
	double	fVoltsTotal[MAX_CHANNELS];			//volts sum used for average caculation
	double	fSD[MAX_CHANNELS][MAX_SDGROUP];		//stardard deviation temp. buffer
	double	fEffectiveBits[MAX_CHANNELS];			//effective bits 
	double	fstDv[MAX_CHANNELS];					//standard derviation
	double	fSDTotal[MAX_CHANNELS];				//standard derviation sum
	UINT	ulSDCount[MAX_CHANNELS];				//standard derviation counts
};

struct SSCAN_OBJ{
	//32 bits counts returned from M40x
	UINT	ulADCounts[MAX_DATA_POINTS];
	//8 bits digital input data from M40x
	BYTE	bDigIn[MAX_DATA_POINTS];	
	//scan mode SINGLE_CHAN_SCAN/MULT_CHAN_CAL_SCAN etc.
	BYTE		bScanMode;			
	//depends on scan mode/cal. scan (0,1,6,7)
	BYTE		bLastScanChan;		
	//SINGLE_CHAN_SCAN mode it is channel ID (0-7)
	BYTE		bSelChanOrNumOfChan;							
	//MULTI_CHAN_SCAN mode it is channel number(1-8)
	//MULTI_CHAN_CAL_SCAN mode it channel number + 1 (2-8)				
	//running average
	BYTE		bChanScanAvg;
	//scan data check sum				
	BYTE		bScanDataChkSum;			
	//check sum error flag	
	BYTE		bChkSumErr;					
	//scan buffer overlapped flag
	BYTE		bScanBuffOverLapped;		
	//AD chip registor corrupted flag
	BYTE		bADChipRegCorrupted;		
	//scan counts since scan started
	unsigned long		ulScanCounts;		
	//Standard deviation data counts in a group
	BYTE		bSDGroup;
	//Scan rate
	double		fScanRate;			
	//When single channel mode it is the same as fScanRate, else it is fScanRate / bSelChanOrNumOfChan / 5
	double		fMultiChanScanRate;			
	//Indicates scan stopped
	BYTE		bScanStopped;
	//When in single channel scan mode & channel is changed, first 4 conversion are invalid
	BYTE		bIsValidData;
	//Based on A/D board range, bADRange = 1(-5V to 5V) or 2(-1V to 1V)
	BYTE		bADRange;
	BYTE		bPlaceholder1;
	BYTE		bPlaceholder2;
	BYTE		bPlaceholder3;
	//6 bytes MAC address
	BYTE		bMACAddr[6];
	UINT		RevDataPoints;
	UINT		uM40xID;		// ID which maps index to an internal dll object. DO NOT MODIFY
	BYTE		bDataReady;		// No longer used, ignore
};

/*****************************************************************************
An INDEX_CONTEXT structure is associated with each index to save GUI settings 
when user switches between indices.
*******************************************************************************/

struct INDEX_CONTEXT{

	CString sIP;			//Ip address of M401 this index is connected to
	CString sSubnetMask;	//Subnet mask
	CString sGateway;		//Gateway 
	BYTE bDhcpTmout;		//DHCP timeout
	CString sEvtLogMsg; //each line in evt log wnd is demarked by a newline.
	BOOLEAN bIsConnected; // is index connected to a M401	
	// so restoring is easy
	BYTE bChanSel;	//Selected channel whose voltage is being displayed
	BYTE bNumOfChan; //Number of channel if in multichannel scan mode
	BYTE bAvg;		 //Average
	BYTE bSaveToFile; // If user is writing scan points to file.
    BYTE bGraphChanSel; //Graph selection, channel whose voltage-graph is being plotted
    BYTE bGraphAC_DC;  //Graph selection, AC - 0 or DC -1
	BYTE bGraphVerticalScale;  //Graph vertical scale 0 - 1uV,1- 10uV, 2-100uV, 3-1mV, 4-10mV,5-100mV, 6-1V
	BYTE bMODE;
	// 0 - Polled mode, 1 - Single Channel scan mode, 
	// 2- Muti channel scan mode, 3 - Multi channel calibration scan mode
	BYTE bDigitalOut;    
	double fScanRate; //Scan rate entered by user
	BYTE bSaveLog;
	// new variables to store extended scan setup
	LONG ExtScan_AD0_Start;       
	LONG ExtScan_AD0_End;         
	LONG ExtScan_AD1_Start;       
	LONG ExtScan_AD1_End;        
	BYTE ExtScan_AD6;      // 0 AD6 not selected, 1 selected
	BYTE ExtScan_AD7;      // 0 AD7 not selected, 1 selected
	BYTE bExtCalScan;      // 1 - extended cal scan, 0 - regular extended cal scan
	double fExScans[400];   // buffer to pass extended scan voltages between dialogs/function
	BYTE bExDigInp[400];    // digital inputs in extended scan
	ULONG ulNumOfExScans;    // number of scans in fExScans, bExDigInp
	BYTE bMuxInverted ;
	ULONG ulExScanCounts ;   //total number of scans received so far
};


/***********************************************************************************
Prototype of functions exported by dll

************************************************************************************/

BOOLEAN Pub_OpenConnection(LPCTSTR lp_addr, int port);
void Pub_CloseConnection(void);
RETURN_CODE Pub_SetRate(double* fRate);
RETURN_CODE Pub_SetChan(BYTE bChan);
RETURN_CODE Pub_EndScan();
RETURN_CODE Pub_SetDigOut(BYTE bDigOutByte);
RETURN_CODE	Pub_SetRTC();
RETURN_CODE Pub_StartScan(BYTE bScanMode,BYTE bChanNum);
RETURN_CODE Pub_GetPolledConv(double* fConvReading,BYTE *bLastDigIn);
RETURN_CODE Pub_OfsCal(double* fOfsReading);
RETURN_CODE Pub_FsCal(double* fFsReading);
RETURN_CODE Pub_GetSelfTestRst(char* bRetString);
RETURN_CODE Pub_SetNewIPAddr(UINT32 ulNewIPAddr, UINT32 ulNewNetMask, UINT32 ulNewGateWay);
BOOLEAN Pub_GetScanData(SSCAN_OBJ* pScanObj);
BOOLEAN Pub_NoCallback(SSCAN_OBJ* pScanObj);
void Pub_ResetScanDataObj(BYTE bScanMode, BYTE bSelChan,BYTE bRunningAvg);
void Pub_ResetMinMax(void);
void Pub_PtrTransfer(SSCAN_VOLTS_BUFF* pScan_v_buff,SSCAN_OBJ* pScanObj);

RETURN_CODE Pub_GetScanTime(SSCAN_OBJ* pScanObj, char *pStartTime, char *pStopTime, double *pfScanTime);
RETURN_CODE Pub_GetTimeDiff(SSCAN_OBJ* pScanObj,ULONG *pDiff);

#endif // _M40XDEFH_
